@@ -8,6 +8,10 @@ import re
 import subprocess
 import sys
 
+if sys.version_info < (3, 0):
+    printf("Error: This script requires Python 3.")
+    sys.exit(1)
+
 # Constants
 VERSION = pkg_resources.get_distribution("publish_npm").version
 DIR = os.getcwd()
@@ -17,10 +21,6 @@ PACKAGE_JSON_PATH = os.path.join(DIR, PACKAGE_JSON)
 
 
 def main():
-    if sys.version_info < (3, 0):
-        printf("Error: This script requires Python 3.")
-        sys.exit(1)
-
     args = parse_command_line_arguments()
 
     # Check to see if the "package.json" file exists
@@ -44,31 +44,35 @@ def main():
         )
 
     # Update the dependencies to the latest versions
-    printf("Updating NPM dependencies...")
-    completed_process = subprocess.run(
-        [
-            "npx",
-            "npm-check-updates",
-            "--upgrade",
-            "--packageFile",
-            PACKAGE_JSON,
-            "--loglevel",
-            "silent",
-        ],
-        shell=True,
-    )
-    if completed_process.returncode != 0:
-        error(
-            'Failed to update the "{}" dependencies to the latest versions.'.format(
-                PACKAGE_JSON
-            )
+    if not args.skip_increment:
+        printf("Updating NPM dependencies...")
+        completed_process = subprocess.run(
+            [
+                "npx",
+                "npm-check-updates",
+                "--upgrade",
+                # Don't upgrade TypeScript, as latest versions will break TSTL
+                "--reject",
+                "typescript",
+                "--packageFile",
+                PACKAGE_JSON,
+                "--loglevel",
+                "silent",
+            ],
+            shell=True,
         )
+        if completed_process.returncode != 0:
+            error(
+                'Failed to update the "{}" dependencies to the latest versions.'.format(
+                    PACKAGE_JSON
+                )
+            )
 
-    # If we updated any dependencies, then we need to install them
-    printf("Installing NPM dependencies...")
-    completed_process = subprocess.run(["npm", "install", "--silent"], shell=True)
-    if completed_process.returncode != 0:
-        error('Failed to run "npm install".')
+        # If we updated any dependencies, then we need to install them
+        printf("Installing NPM dependencies...")
+        completed_process = subprocess.run(["npm", "install", "--silent"], shell=True)
+        if completed_process.returncode != 0:
+            error('Failed to run "npm install".')
 
     # Before we increment the version number, make sure that the program compiles
     if is_typescript_project():
@@ -76,10 +80,10 @@ def main():
         compile_typescript()
 
     # Increment the version number
-    if args.skip_increment:
-        version = get_version_from_package_json()
-    else:
-        version = increment_version_in_package_json()
+    version = get_version_from_package_json()
+    if not args.skip_increment:
+        version = increment_version(version)
+        put_version(version)
 
     # Build the program again so that the new version number is included in the compiled code
     if is_typescript_project():
@@ -128,6 +132,13 @@ def parse_command_line_arguments():
         ),
     )
 
+    parser.add_argument(
+        "-u",
+        "--skip-update",
+        action="store_true",
+        help='do not update NPM dependencies in the "{}" file'.format(PACKAGE_JSON),
+    )
+
     return parser.parse_args()
 
 
@@ -141,25 +152,23 @@ def get_version_from_package_json():
     return package_json["version"]
 
 
-def increment_version_in_package_json():
-    with open(PACKAGE_JSON_PATH, "r") as file_handle:
-        package_json = json.load(file_handle)
-
-    if "version" not in package_json:
-        error('Failed to find the version in the "{}" file.'.format(PACKAGE_JSON_PATH))
-
-    match = re.search(r"(.+\..+\.)(.+)", package_json["version"])
+def increment_version(version: str):
+    match = re.search(r"(.+\..+\.)(.+)", version)
     if not match:
-        error(
-            'Failed to parse the version number of "{}".'.format(
-                package_json["version"]
-            )
-        )
+        error('Failed to parse the version number of "{}".'.format(version))
     version_prefix = match.group(1)
     patch_version = int(match.group(2))  # i.e. the third number
     incremented_patch_version = patch_version + 1
     incremented_version = version_prefix + str(incremented_patch_version)
-    package_json["version"] = incremented_version
+
+    return incremented_version
+
+
+def put_version(version: str):
+    with open(PACKAGE_JSON_PATH, "r") as file_handle:
+        package_json = json.load(file_handle)
+
+    package_json["version"] = version
 
     with open(PACKAGE_JSON_PATH, "w", newline="\n") as file_handle:
         json.dump(package_json, file_handle, indent=2, separators=(",", ": "))
@@ -169,8 +178,6 @@ def increment_version_in_package_json():
     if completed_process.returncode != 0:
         error('Failed to sort the "{}" file.'.format(PACKAGE_JSON))
 
-    return incremented_version
-
 
 def is_typescript_project():
     with open(PACKAGE_JSON_PATH, "r") as file_handle:
@@ -191,6 +198,10 @@ def compile_typescript():
         if completed_process.returncode != 0:
             error('Failed to run the "build.sh" script.')
     else:
+        completed_process = subprocess.run(["rm", "-rf", "dist"], shell=True)
+        if completed_process.returncode != 0:
+            error('Failed to remove the "dist" directory.')
+
         completed_process = subprocess.run(["npx", "tsc"], shell=True)
         if completed_process.returncode != 0:
             error('Failed to build the project with "npx tsc".')
@@ -209,8 +220,21 @@ def git_commit_if_changes(version):
     # Check to see if there are any changes
     # https://stackoverflow.com/questions/3878624/how-do-i-programmatically-determine-if-there-are-uncommitted-changes
     completed_process = subprocess.run(["git", "diff-index", "--quiet", "HEAD", "--"])
-    if completed_process.returncode == 0:
-        # There are no changes
+    changes_to_existing_files = completed_process.returncode != 0
+
+    # Check to see if there are any untracked files
+    # https://stackoverflow.com/questions/11021287/git-detect-if-there-are-untracked-files-quickly
+    completed_process = subprocess.run(
+        ["git", "ls-files", "--other", "--directory", "--exclude-standard"],
+        stdout=subprocess.PIPE,
+    )
+    if completed_process.returncode != 0:
+        error("Failed to git ls-files.")
+    git_output = completed_process.stdout.decode("utf-8")
+    untracked_files_exist = git_output is not None and git_output.strip() != ""
+
+    if not changes_to_existing_files and not untracked_files_exist:
+        printf("There are no changes to push to git.")
         return
 
     # Commit to the repository
@@ -228,6 +252,8 @@ def git_commit_if_changes(version):
     if completed_process.returncode != 0:
         error("Failed to git push.")
 
+    printf('Pushed a commit to git for version "{}".'.format(version))
+
 
 def error(msg):
     printf("Error: {}".format(msg))
